import * as React from 'react'
import * as PropTypes from 'prop-types'
import styled from 'styled-components'
import docgen from '../../docgen.json'

const Base = styled('table')({
  borderCollapse: 'collapse',
  border: 0,
  display: 'block',
  '@media (min-width: 720px)': {
    display: 'table',
    margin: '0 -0.5rem',
  },
})

const HeaderCellGroup = styled('thead')({
  display: 'none',
  '@media (min-width: 720px)': {
    display: 'table-header-group',
  },
})

const HeaderRow = styled('tr')({
  display: 'block',
  margin: '2rem 0',
  '@media (min-width: 720px)': {
    display: 'table-row',
    margin: 0,
  },
})

const HeaderCell = styled('th')({
  verticalAlign: 'top',
  textAlign: 'left',
  fontSize: '0.75rem',
  fontWeight: 'bolder',
  textTransform: 'uppercase',
  border: 0,
  display: 'block',
  padding: 0,
  '@media (min-width: 720px)': {
    display: 'table-cell',
    padding: '0 0.5rem',
  },
})

const MainBodyCell = styled('th')({
  verticalAlign: 'top',
  textAlign: 'left',
  border: 0,
  lineHeight: 1,
  display: 'block',
  padding: 0,
  '@media (min-width: 720px)': {
    display: 'table-cell',
    padding: '0.5rem',
  },
})

const BodyCellGroup = styled('tbody')({
  display: 'block',
  '@media (min-width: 720px)': {
    display: 'table-row-group',
  },
})

const BodyCell = styled('td')({
  verticalAlign: 'top',
  padding: 0,
  border: 0,
  display: 'block',
  '&[data-column-name]::before': {
    display: 'inline',
    fontSize: '0.75rem',
    fontWeight: 'bolder',
    textTransform: 'uppercase',
    content: "attr(data-column-name) ':'",
    marginRight: '1rem',
  },
  '@media (min-width: 720px)': {
    padding: '0.5rem 0.5rem',
    display: 'table-cell',
    lineHeight: 1.25,
    '&[data-column-name]::before': {
      display: 'none',
    },
  },
})

const Code = styled('code')({
  display: 'inline-block',
})

const propTypes = {
  of: PropTypes.string,
}

type Props = PropTypes.InferProps<typeof propTypes>

type Docs = {
  displayName: string,
  description: string,
  methods: unknown[],
  props: {
    [name: string]: {
      required: boolean,
      type: {
        name: string,
        value: { value: string }[]
      },
      defaultValue: { value: string, },
      description: string,
    }
  }
}

const Props: React.FC<Props> = ({ of: ofAttr }) => {
  const docs = (docgen as unknown as Docs[]).find(d => d.displayName === ofAttr)

  if (!(docs as Docs)) {
    return null
  }

  return (
    <Base>
      <colgroup>
        <col width="20%" />
        <col width="25%" />
        <col width="25%" />
        <col width="*" />
      </colgroup>
      <HeaderCellGroup>
        <HeaderRow>
          <HeaderCell>
            Name
          </HeaderCell>
          <HeaderCell>
            Type
          </HeaderCell>
          <HeaderCell>
            Default
          </HeaderCell>
          <HeaderCell>
            Description
          </HeaderCell>
        </HeaderRow>
      </HeaderCellGroup>
      <BodyCellGroup>
        {
          Object.entries((docs as Docs).props).map(([name, def]) => (
            <HeaderRow>
              <MainBodyCell>
                <Code>
                  {def.required ? name : name + '?'}
                </Code>
              </MainBodyCell>
              <BodyCell
                data-column-name="Type"
              >
                {
                  def.type.name === 'enum'
                  && (
                    <Code>
                      {def.type.value.map(v => v.value).join(' | ')}
                    </Code>
                  )
                }
                {
                  def.type.name !== 'enum'
                  && (
                    <Code>
                      {def.type.name}
                    </Code>
                  )
                }
              </BodyCell>
              <BodyCell
                data-column-name={def.defaultValue ? 'Default' : undefined }
              >
                {
                  def.defaultValue
                  && (
                    <Code>
                      {JSON.stringify(def.defaultValue.value)}
                    </Code>
                  )
                }
              </BodyCell>
              <BodyCell>
                {def.description}
              </BodyCell>
            </HeaderRow>
          ))
        }
      </BodyCellGroup>
    </Base>
  )
}

Props.propTypes = propTypes

export default Props
